// Craig S. Kaplan
//
// Draw a circle fractal like the one in Learning Processing, 
// Page 256.  This version uses the added sophistication of 
// embedding recursive calls inside geometric contexts, which
// is a good jumping off point for more complex recursive 
// designs.
//
// Note that because this version uses scale() to change the 
// sizes of circles, the stroke widths will be scaled too.

// Move the number of recursive levels into a global variable,
// so that we can manipulate it interactively from the keyboard.
int levels = 8;

// Don't draw anything in the setup() function now, since
// we may want to change the number of recursive levels
// frame-to-frame.
void setup()
{
  size( 1050, 600 );
}

// Because the drawing will be done inside a geometric
// context, there's no need for lots of parameters in this
// version of the function.  All we need to keep track of
// is the number of recursive levels.
void drawCircles( int levels )
{
  ellipse( 0, 0, 500, 500 );

  if ( levels > 0 ) {
    // Draw the recursive circles on the left.
    pushMatrix();
    translate( -250, 0 );
    scale( 0.5 );
    drawCircles( levels - 1 );
    popMatrix();

    // Draw the recursive circles on the right.
    pushMatrix();
    translate( 250, 0 );
    scale( 0.5 );
    drawCircles( levels - 1 );
    popMatrix();
  }
}

void draw()
{
  background( 150 );
  
  // Set up some pretty grey levels and partial transparency.
  // Kind of a nice effect.
  stroke( 0 );
  strokeWeight( 5 );
  fill( 255, 60 );

  // Set up the initial geometric context so that
  // the entire fractal is visible.
  pushMatrix();
  translate( width/2, height/2 );
  drawCircles( levels );
  popMatrix();
}

void keyPressed()
{
  if( keyCode == LEFT ) {
    levels = max( 0, levels - 1 );
  } else if( keyCode == RIGHT ) {
    ++levels;
  }
}