// Craig S. Kaplan
// 
// This is a longer sketch that we're offering to you because
// it might be fun to play with.  You don't have to understand
// all the details in this sketch, though there's nothing here
// that hasn't already been taught in CS 106, so it's within your
// grasp.  
//
// The sketch lets you create fractals using images, by placing 
// copies of an image and setting the translation, rotation and
// scale of each copy interactively.  The default hand image
// lets you position smaller hands at the end of each finger, 
// creating a "hand tree" thing.

import controlP5.*;

ControlP5 ui;
Button add;
Button show_outlines;
Slider levs;

PImage hand;

PlacedImage main_img;
PlacedImage[] copies;

PlacedImage dragging;
int mode;

void setup()
{
  size( 800, 600 );
  hand = loadImage( "hand2.png" );

  //hand = loadImage( "giraffe.png" );
  main_img = new PlacedImage( hand );
  copies = new PlacedImage[0];

  draw_bounds = true;

  ui = new ControlP5( this );
  add = ui.addButton( "New Copy" )
    .setPosition( 610, 20 );
  show_outlines = ui.addButton( "Show Outlines" )
    .setPosition( 610, 50 )
    .setSwitch( true )
    .setOff();
  levs = ui.addSlider( "Levels" )
    .setRange( 1, 6 )
    .setDecimalPrecision( 0 )
    .setPosition( 610, 80 )
    .setSize( 150, 20 )
    .setValue( 1 );
}

void drawFractal( int levs )
{
  main_img.drawSelf();
  
  if( levs > 0 ) {
    for( PlacedImage im : copies ) {
      PMatrix2D M = new PMatrix2D( main_img.M );
      M.invert();
      pushMatrix();
      applyMatrix( im.M );
      applyMatrix( M );
      drawFractal( levs - 1 );
      popMatrix();
    }
  }
}

void draw()
{
  background( 50 );
  // background( 255 );

  draw_bounds = show_outlines.isOn();

  drawFractal( (int)levs.getValue() );

  fill( 80 );
  noStroke();
  rect( 600, 0, 200, 600 );
}

void mousePressed()
{
  dragging = null;
  mode = DRAG_NOTHING;

  int d = main_img.hitTest( mouseX, mouseY );
  if( d != DRAG_NOTHING );
  dragging = main_img;
  mode = d;
  
  for ( PlacedImage im : copies ) {
    d = im.hitTest( mouseX, mouseY );
    if ( d != DRAG_NOTHING ) {
      dragging = im;
      mode = d;
    }
  }
}

void mouseDragged()
{
  if ( dragging != null ) {
    dragging.drag( mode, pmouseX, pmouseY, mouseX, mouseY );
  }
}

void mouseReleased()
{
  dragging = null;
  mode = DRAG_NOTHING;
}

void controlEvent( ControlEvent ce )
{
  if ( ce.isFrom( add ) ) {
    copies = (PlacedImage[])append(
      copies, new PlacedImage( hand ) );
  }
}