class Drawing
{
  PShape[] shapes;
  PlacedDrawing[] children;
  
  Drawing()
  {
    shapes = new PShape[0];
    children = new PlacedDrawing[0];
  }
  
  void draw( int depth )
  {
    for( PShape shp : shapes ) {
      fill( 0 );
      noStroke();
      shp.disableStyle();
      shape( shp, 0, 0 );
    }
    
    if( depth > 0 ) {
      for( PlacedDrawing pd : children ) {
        pd.draw( depth );
      }
    }
  }
  
  void addShape( PShape shp )
  {
    shapes = (PShape[])append( shapes, shp );
  }
}

class PlacedDrawing
{
  float tx;
  float ty;
  float sx;
  float sy;
  float theta;
  
  Drawing drawing;
  
  PlacedDrawing( Drawing ch )
  {
    drawing = ch;
    tx = 0.0;
    ty = 0.0;
    sx = 1.0;
    sy = 1.0;
    theta = 0.0;
  }
  
  void draw( int depth )
  {
    pushMatrix();
    translate( tx, ty );
    rotate( theta );
    scale( sx, sy );
    drawing.draw( depth - 1 );
    popMatrix();
  }
}

Drawing[] drawings;
float thumb_scroll;

void setup()
{
  size( 810, 640 );
  
  drawings = new Drawing[2];
  
  Drawing sq = new Drawing();
  sq.addShape( createShape( RECT, 200, 200, 200, 200 ) );
  drawings[0] = sq;
  
  Drawing el = new Drawing();
  el.addShape( createShape( ELLIPSE, 300, 300, 200, 200 ) );
  drawings[1] = el;
  
  thumb_scroll = 0.0;
}

void drawThumbs()
{
  for( int idx = 0; idx < drawings.length; ++idx ) {
    noStroke();
    fill( 255 );
    rect( 20, 20 + idx * 170, 150, 150 );
    clip( 20, 20 + idx * 170, 150, 150 );
    
    pushMatrix();
    translate( 20, 20 + idx * 170 );
    scale( 0.25, 0.25 );
    drawings[idx].draw( 5 );
    popMatrix();
    
    noClip();
  }
}

void draw()
{
  background( 50 );
  
  fill( 255 );
  noStroke();
  rect( 190, 20, 600, 600 );
  drawThumbs();
}