// Craig S. Kaplan
//
// Draw the Sierpinski Carpet, a well known fractal
// design.  Use for loops in the recursive case to 
// make calls on all squares in a 3x3 grid except
// for the central cell.

int levels;

void drawCarpet( int levels )
{
  if( levels == 0 ) {
    // The base case: just draw a square.
    rect( 0, 0, 300, 300 );
  } else {
    // The recursive case: make a 3x3 grid
    // and make recursive calls on all eight
    // apart from the central cell.
    for( int row = 0; row < 3; ++row ) {
      for( int col = 0; col < 3; ++col ) {
        if( row == 1 && col == 1 ) {
          continue;
        }
        pushMatrix();
        translate( col * 100, row * 100 );
        scale( 1.0 / 3.0 );
        drawCarpet( levels - 1 );
        popMatrix();
      }
    }
  }
}

void setup()
{
  size( 650, 650 );
}

void draw()
{
  background( 255 );
  fill( 0 );
  noStroke();
  pushMatrix();
  translate( 25, 25 );
  scale( 2 );
  drawCarpet( levels );
  popMatrix();
}

void keyPressed()
{
  if( keyCode == LEFT ) {
    levels = max( 0, levels - 1 );
  } else if( keyCode == RIGHT ) {
    ++levels;
  }
}