// Craig S. Kaplan
//
// Draw the Sierpinski Gasket, one of the most famous
// self-similar fractal shapes.  Use the method where
// every recursive call is placed inside a geometric
// context.

int levels = 1;

// A helper function for the base case, just so that
// we can figure out the coordinates in a separate place.
void drawTriangle()
{
  triangle( 0, 0, -50, 50*sqrt(3), 50, 50*sqrt(3) );
}

void drawGasket( int levels )
{
  if ( levels == 0 ) {
    drawTriangle();
  } else {
    // Make three recursive calls, each inside its
    // own geometric contexts.
    
    pushMatrix();
    scale( 0.5 );
    drawGasket( levels - 1 );
    popMatrix();

    pushMatrix();
    translate( -25, 25*sqrt(3) );
    scale( 0.5 );
    drawGasket( levels - 1 );
    popMatrix();

    pushMatrix();
    translate( 25, 25*sqrt(3) );
    scale( 0.5 );
    drawGasket( levels - 1 );
    popMatrix();
  }
}

void setup()
{
  size( 500, 500 );
}

void draw()
{ 
  background( 200 );
  pushMatrix();
  translate( 250, 0 );
  scale( 5 );

  // Always draw a black triangle
  // in the background.
  fill( 0 );
  noStroke();
  drawGasket( 0 );

  fill( 150 );
  drawGasket( levels );
  popMatrix();
}

void keyPressed()
{
  if( keyCode == LEFT ) {
    levels = max( 0, levels - 1 );
  } else if( keyCode == RIGHT ) {
    ++levels;
  }
}