// Craig S. Kaplan
//
// Draw the Sierpinski Gasket, one of the most famous
// self-similar fractal shapes.  This version is based on
// keeping track of the three corners of a triangle as
// proper points (using the PVector class).  To make a 
// recursive call, we figure out the corners of the three
// recursive triangles by using a helper function that
// computes the midpoint (average) between two other points.

int levels = 1;
int numtris;

// A helper function to compute the midpoint of two points.
// the X and Y coordinates of the midpoint are simply the
// averages of the X and Y coordinates of the two starting
// points.
PVector mid( PVector P, PVector Q )
{
  return new PVector( 
    lerp( P.x, Q.x, 0.5 ),
    lerp( P.y, Q.y, 0.5 ) );
}

// A recursive function to draw the Sierpinski Gasket.
void drawGasket( int levels, PVector a, PVector b, PVector c )
{
  if( levels == 0 ) {
    // Base case: just draw a triangle.
    triangle( a.x, a.y, b.x, b.y, c.x, c.y );
    ++numtris;
  } else {
    // Recursive case: find the midpoints you need to draw
    // three recursive copies of the gasket.
    drawGasket( levels - 1, mid( a, b ), mid( b, c ), b );
    drawGasket( levels - 1, mid( a, b ), mid( a, c ), a );
    drawGasket( levels - 1, mid( a, c ), mid( b, c ), c );
  }
}

void setup()
{
  size( 500, 433 );
}

void draw()
{ 
  numtris = 0;
  background( 200 );
  fill( 0 );
  noStroke();
  
  // Carefully choose the three starting points
  // so that the fractal fits into an equilateral
  // triangle.
  drawGasket( levels, 
    new PVector( 250, 0 ),
    new PVector( 0, 433 ),
    new PVector( 500, 433 ) );
}

void keyPressed()
{
  if( keyCode == LEFT ) {
    levels = max( 0, levels - 1 );
  } else if( keyCode == RIGHT ) {
    ++levels;
  } else if( key == ' ' ) {
    println( numtris );
  }
}